@@ -20,7 +20,6 @@
 
 #include "../command.h"
 
-static char *opt_session_name;
 static int opt_no_wait;
 static struct mi_writer *writer;
 
@@ -82,19 +81,23 @@ end:
 /*
  * Start tracing for all trace of the session.
  */
-static int stop_tracing(void)
+static int stop_tracing(const char *arg_session_name)
 {
 	int ret;
 	char *session_name;
 
-	if (opt_session_name == NULL) {
+	if (arg_session_name == NULL) {
 		session_name = get_session_name();
-		if (session_name == NULL) {
-			ret = CMD_ERROR;
-			goto error;
-		}
 	} else {
-		session_name = opt_session_name;
+		session_name = strdup(arg_session_name);
+		if (session_name == NULL) {
+			PERROR("Failed to copy session name");
+		}
+	}
+
+	if (session_name == NULL) {
+		ret = CMD_ERROR;
+		goto error;
 	}
 
 	ret = lttng_stop_tracing_no_wait(session_name);
@@ -145,9 +148,7 @@ static int stop_tracing(void)
 	}
 
 free_name:
-	if (opt_session_name == NULL) {
-		free(session_name);
-	}
+	free(session_name);
 
 error:
 	return ret;
@@ -162,6 +163,7 @@ int cmd_stop(int argc, const char **argv)
 {
 	int opt, ret = CMD_SUCCESS, command_ret = CMD_SUCCESS, success = 1;
 	static poptContext pc;
+	const char *arg_session_name = NULL;
 	const char *leftover = NULL;
 
 	pc = poptGetContext(NULL, argc, argv, long_options, 0);
@@ -217,7 +219,7 @@ int cmd_stop(int argc, const char **argv)
 		}
 	}
 
-	opt_session_name = (char*) poptGetArg(pc);
+	arg_session_name = poptGetArg(pc);
 
 	leftover = poptGetArg(pc);
 	if (leftover) {
@@ -226,7 +228,7 @@ int cmd_stop(int argc, const char **argv)
 		goto end;
 	}
 
-	command_ret = stop_tracing();
+	command_ret = stop_tracing(arg_session_name);
 	if (command_ret) {
 		success = 0;
 	}
