@@ -179,6 +179,24 @@ const struct lttng_evaluation *lttng_notification_get_evaluation(
 	return notification ? notification->evaluation : NULL;
 }
 
+const struct lttng_condition *lttng_notification_get_const_condition(
+		const struct lttng_notification *notification)
+{
+	return notification ? lttng_trigger_get_const_condition(notification->trigger) : NULL;
+}
+
+const struct lttng_evaluation *lttng_notification_get_const_evaluation(
+		const struct lttng_notification *notification)
+{
+	return notification ? notification->evaluation : NULL;
+}
+
+const struct lttng_trigger *lttng_notification_get_const_trigger(
+		const struct lttng_notification *notification)
+{
+	return notification ? notification->trigger : NULL;
+}
+
 const struct lttng_trigger *lttng_notification_get_trigger(
 		struct lttng_notification *notification)
 {
